import subprocess

import bs4
import requests

from .constants import *


def test(args):
    contest, problem_letter = translate_problem_name(args.problem)
    problem = contest + problem_letter

    if not os.path.exists('in'):
        os.makedirs('in')
    if not os.path.exists('ans'):
        os.makedirs('ans')

    if args.download:
        for file in os.listdir('in'):
            os.remove(os.path.join('in', file))
        for file in os.listdir('ans'):
            os.remove(os.path.join('ans', file))

    if len(os.listdir('in')) == 0:
        r = requests.get(f'https://codeforces.com/problemset/problem/{contest}/{problem_letter}')
        try:
            r.raise_for_status()
        except requests.HTTPError:
            print(error_style('Something went wrong while downloading tests'))
            sys.exit()

        soup = bs4.BeautifulSoup(r.text, 'html.parser')
        tests_input = soup.select('div.sample-test div.input pre')
        tests_answer = soup.select('div.sample-test div.output pre')
        for i, (test_in, test_ans) in enumerate(zip(tests_input, tests_answer), start=1):
            with open(os.path.join('in', f'{i}.in'), 'w') as input_file:
                input_file.write(test_in.string.lstrip())
            with open(os.path.join('ans', f'{i}.out'), 'w') as answer_file:
                answer_file.write(test_ans.string.strip())

    compile_command = get_compile_command()
    if compile_command:
        if compile_solution(problem, compile_command).returncode != 0:
            print(error_style('Solution has not been compiled.'))
            sys.exit()
        else:
            print('Solution has been compiled.')

    i = 1
    while os.path.exists(os.path.join('in', f'{i}.in')):
        test_solution(problem, i)
        i += 1


def compile_solution(problem, compile_command):
    language = get_language()
    try:
        if language.ext in ('c', 'cpp', 'kt'):
            return subprocess.run([*compile_command.split(' '), f'{problem}.{language.ext}', '-o', problem], timeout=10)
        else:
            return subprocess.run([*compile_command.split(' '), f'{problem}.{language.ext}'], timeout=10)
    except subprocess.TimeoutExpired:
        print(error_style('Compilation time has exceeded 10 seconds.'))
        print('Make sure you are using appropriate compile and run commands for your language.')
        sys.exit()
    except OSError:
        print(error_style('Compile command is wrong or compiler is not installed.'))
        sys.exit()


def test_solution(problem, i):
    with open(os.path.join('in', f'{i}.in')) as input_file:
        test_in = input_file.read()
    with open(os.path.join('ans', f'{i}.out')) as answer_file:
        test_ans = answer_file.read()

    language = get_language()
    run_command = get_run_command()
    try:
        if run_command:
            if language.ext == 'java':
                r = subprocess.run([*run_command.split(' '), problem], input=test_in, capture_output=True,
                                   timeout=10, encoding='utf-8')
            else:
                r = subprocess.run([*run_command.split(' '), f'{problem}.{language.ext}'], input=test_in,
                                   capture_output=True, timeout=10, encoding='utf-8')
        else:
            r = subprocess.run('./' + problem, input=test_in, capture_output=True, timeout=10, encoding='utf-8')
        test_out = r.stdout.strip()
        test_err = r.stderr.strip()
    except FileNotFoundError:
        print(error_style('Executable file has not been found.'))
        print('Make sure you are using appropriate compile and run commands for your language.')
        sys.exit()
    except subprocess.TimeoutExpired:
        print(negative_style('Execution time exceeded 10 seconds.'))
        return

    if test_out.split() == test_ans.split():    # delete every whitespace
        print(positive_style('Test passed'))
    else:
        print(negative_style('Test did not pass'))
        print('Program output:', test_out, sep='\n')
        if test_err:
            print('\nProgram error:', test_err, sep='\n')
        print('\nAnswer:', test_ans, sep='\n')



import getpass

import keyring

from .constants import *


def config(args):
    print('Choose one of the following (type an integer):',
          '  1. change the template file',
          '  2. change username and password',
          '  3. change password',
          '  4. change language',
          '  5. set compile command',
          '  6. set run command',
          sep='\n')
    try:
        while (choice := input()) not in {'1', '2', '3', '4', '5', '6'}:
            print(warning_style('Type an integer 1-6:'))
        choice = int(choice)
    except KeyboardInterrupt:
        print(warning_style('Aborted.'))
        sys.exit(0)

    if not os.path.exists(os.path.join(os.path.expanduser("~"), '.codeforces-toolbox')):
        os.makedirs(os.path.join(os.path.expanduser("~"), '.codeforces-toolbox'))

    try:
        config_dict = json.load(open(CONFIG_FILE))
    except (FileNotFoundError, json.decoder.JSONDecodeError):
        config_dict = dict()

    try:
        if choice == 1:
            config_dict['template'] = input('Path to the template: ')

        if choice == 2:
            config_dict['username'] = input('Username: ')

        if choice in (2, 3):
            try:
                username = config_dict['username']
            except KeyError:
                print(error_style('First enter your username.'))
                sys.exit()
            password = getpass.getpass('Password: ')
            keyring.set_password('codeforces-toolbox', username, password)

        if choice == 4:
            print('Change your programming language and the program that Codeforces will use to run your solution.')
            print('Choose one of the following (type an integer):')
            for key, lan in LANGUAGES.items():
                print(f'    {key + "." :3} {lan.name:30}' + neutral_style(lan.ext))
            while (language := input()) not in LANGUAGES.keys():
                print(warning_style('Type an integer 1-5:'))
            config_dict['language'] = (LANGUAGES[language].n, LANGUAGES[language].name, LANGUAGES[language].ext)

        if choice == 5:
            print('Set compile command, e.g. `g++ -Wall -O1`.')
            print('If you are using ' + neutral_style('Python') +
                  ' or do not want to compile your solutions, just press enter.')
            config_dict['compile'] = input('Compile command: ')

        if choice == 6:
            print('Set run command, e.g. `python` or enter an absolute path to the interpreter.')
            print('If you are using ' + neutral_style('C++') + ' or ' + neutral_style('C') +
                  ' you can just press enter - your run command will be just `./`.')
            config_dict['run'] = input('Run command: ')

    except KeyboardInterrupt:
        print(warning_style('\nAborted.'))

    finally:
        json.dump(config_dict, open(CONFIG_FILE, 'w'))



import shutil

import bs4
import requests

from .constants import *


def race(args):
    language = get_language()
    contest = args.contest
    template = get_template()
    if not os.path.exists(template):
        print(error_style('Template file does not exist.'))
        sys.exit()
    try:
        os.makedirs(contest)
    except OSError:
        print(error_style(f'Folder {contest} already exists.'))
        sys.exit()
    os.chdir(contest)
    for problem_letter in contest_letters(contest):
        shutil.copy(template, f'{contest}{problem_letter}.{language.ext}')


def contest_letters(contest):
    r = requests.get(f'https://codeforces.com/contest/{contest}')
    try:
        r.raise_for_status()
        if len(r.history):
            raise requests.HTTPError
    except requests.HTTPError:
        print(warning_style('Something went wrong while accessing problems'))
        return contest_letters_default(contest)
    soup = bs4.BeautifulSoup(r.text, 'html.parser')
    letters = soup.select('table.problems tr:nth-child(n+2) td:first-child a')
    return [letter.text.strip() for letter in letters]


def contest_letters_default(contest):
    d = input('Do you want to create default A-G files? [y/n] ')
    if d.lower() in ('y', 'yes'):
        return list('ABCDEFG')
    else:
        print('Aborted')
        os.chdir('..')
        os.rmdir(contest)
        sys.exit()



import json
import os
import sys

from sty import fg, rs


class Language:
    def __init__(self, n, name, extension):
        self.n = n    # value in CF submit request
        self.name = name
        self.ext = extension


CONFIG_FILE = os.path.join(os.path.expanduser("~"), '.codeforces-toolbox', 'cft_config.json')

LANGUAGES = {
    '1': Language(42, 'GNU C++11', 'cpp'),
    '2': Language(50, 'GNU C++14', 'cpp'),
    '3': Language(54, 'GNU C++17', 'cpp'),
    '4': Language(61, 'GNU C++17 (64 bit)', 'cpp'),
    '5': Language(2,  'Microsoft Visual C++ 2010', 'cpp'),
    '6': Language(59, 'Microsoft Visual C++ 2017', 'cpp'),
    '7': Language(43, 'GNU C11', 'c'),
    '8': Language(7,  'Python 2.7', 'py'),
    '9': Language(31, 'Python 3.8', 'py'),
    '10': Language(40, 'PyPy 2.7', 'py'),
    '11': Language(41, 'PyPy 3.7', 'py'),
    '12': Language(36, 'Java 8', 'java'),
    '13': Language(60, 'Java 11', 'java'),
    '14': Language(48, 'Kotlin', 'kt'),
    '15': Language(49, 'Rust', 'rs'),
    '16': Language(9, 'C# Mono', 'cs'),
    '17': Language(65, 'C# .NET', 'cs'),
    '18': Language(32, 'Go', 'go')
}


def error_style(message):
    return fg(196) + message + rs.all   # red


def warning_style(message):
    return fg(220) + message + rs.all   # yellow


def positive_style(message):
    return fg(70) + message + rs.all    # green


def negative_style(message):
    return fg(208) + message + rs.all   # orange


def neutral_style(message):
    return fg(69) + message + rs.all    # blue


def translate_problem_name(problem):
    if len(problem) <= 2:
        problem_letter = problem
        contest = os.path.basename(os.getcwd())
    else:
        problem_letter = problem[4:]
        contest = problem[:4]
    return contest, problem_letter


def get_template():
    try:
        config_dict = json.load(open(CONFIG_FILE))
        template = config_dict['template']
    except FileNotFoundError:
        print(error_style('Configuration file has not been found.'))
        sys.exit()
    except KeyError:
        print(error_style('Specify your template file first.'))
        sys.exit()
    return template


def get_username():
    try:
        config_dict = json.load(open(CONFIG_FILE))
        username = config_dict['username']
    except FileNotFoundError:
        print(error_style('Configuration file has not been found.'))
        sys.exit()
    except KeyError:
        print(error_style('Specify your username first.'))
        sys.exit()
    return username


def get_language():
    try:
        config_dict = json.load(open(CONFIG_FILE))
        language = config_dict['language']
    except FileNotFoundError:
        print(error_style('Configuration file has not been found.'))
        sys.exit()
    except KeyError:
        print(error_style('Specify your programming language first.'))
        sys.exit()
    return Language(*language)


def get_run_command():
    try:
        config_dict = json.load(open(CONFIG_FILE))
        run_command = config_dict['run']
    except FileNotFoundError:
        print(error_style('Configuration file has not been found.'))
        sys.exit()
    except KeyError:
        return ''
    return run_command


def get_compile_command():
    try:
        config_dict = json.load(open(CONFIG_FILE))
        compile_command = config_dict['compile']
    except FileNotFoundError:
        print(error_style('Configuration file has not been found.'))
        sys.exit()
    except KeyError:
        return ''
    return compile_command
